//////////////////////////////////////////////////////////////////////////
// File: 	ch4b.cpp                                            	//
// Description: A program that displays multiple sprites		//
// Author:	gbajunkie 	  					//
// Date: 	9th April 2002                                      	//
//////////////////////////////////////////////////////////////////////////

#include "gba.h"		//GBA register definitions
#include "keypad.h"     	//button registers
#include "dispcnt.h"    	//REG_DISPCNT register #define
#include "ch4b.h"               //my generic sprite header file
#include "palette.h"  		//256 colour palette array for all sprites
#include "pacsprite.h"          //pacman sprite image data
#include "ghostsprite.h"        //ghost sprite image data

//create an OAM variable and make it point to the address of OAM
u16* OAM = (u16*)0x7000000;

//create the array of sprites (128 is the maximum)
OAMEntry sprites[128];

//create the rotation and scaling array (overlaps the OAMEntry array memory)
pRotData rotData = (pRotData)sprites;

//Copy our sprite array to OAM
void CopyOAM()
{
	u16 loop;
	u16* temp;
	temp = (u16*)sprites;
	for(loop = 0; loop < 128*4; loop++)
	{
		OAM[loop] = temp[loop];
	}
}

//Set sprites to off screen
void InitializeSprites()
{
	u16 loop;
	for(loop = 0; loop < 128; loop++)
	{
		sprites[loop].attribute0 = 160;  //y to > 159
		sprites[loop].attribute1 = 240;  //x to > 239
	}
}

//wait for the screen to stop drawing
void WaitForVsync()
{
	while((volatile u16)REG_VCOUNT != 160){}
}


int AgbMain()
{
	u16 loop;       //generic loop variable
        s16 x = 100;	//variables to hold position of pacman sprite on screen
	s16 y = 60;
	s16 x2 = 50;	//variables to hold position of ghost sprite on screen
	s16 y2 = 100;

       	SetMode(MODE_1 | OBJ_ENABLE | OBJ_MAP_1D); //set mode 1 and enable sprites and 1d mapping

	for(loop = 0; loop < 256; loop++)          //load the palette into memory
		OBJPaletteMem[loop] = palette[loop];

	InitializeSprites();                       //set all sprites off screen (stops artifact)

  	sprites[0].attribute0 = COLOR_256 | SQUARE | y;	//setup sprite info, 256 colour, shape and y-coord
        sprites[0].attribute1 = SIZE_16 | x;		//size 16x16 and x-coord
        sprites[0].attribute2 = 0;                      //pointer to tile where sprite starts

        for(loop = 0; loop < 128; loop++)               //load 1st sprite image data
       	{
       		OAMData[loop] = pacspriteData[loop];
       	}

        sprites[1].attribute0 = COLOR_256 | SQUARE | y2;//setup sprite info, 256 colour, shape and y-coord
        sprites[1].attribute1 = SIZE_16 | x2;           //size 16x16 and x-coord
        sprites[1].attribute2 = 8;                      //pointer to tile where sprite starts

       	for(loop = 128; loop < 256; loop++)             //load 2nd sprite image data
       	{
       		OAMData[loop] = ghostspriteData[loop-128];
       	}

        while(1)                                //main loop
        {
		WaitForVsync();			//waits for the screen to stop drawing
		CopyOAM();			//Copies sprite array into OAM.
	}
}

