//////////////////////////////////////////////////////////////////////////
// File: 	ch4c.cpp                                            	//
// Description: A program that displays an animated sprite		//
// Author:	gbajunkie 	  					//
// Date: 	9th April 2002                                      	//
//////////////////////////////////////////////////////////////////////////

#include "gba.h"	//GBA register definitions
#include "keypad.h"     //button registers
#include "dispcnt.h"    //REG_DISPCNT register #defines
#include "ch4c.h"       //my generic sprite header file
#include "palette.h"  	//256 colour palette array for all sprites
#include "pacanim.h"    //pacman sprite image data

//create an OAM variable and make it point to the address of OAM
u16* OAM = (u16*)0x7000000;

//create the array of sprites (128 is the maximum)
OAMEntry sprites[128];

//create the rotation and scaling array (overlaps the OAMEntry array memory)
pRotData rotData = (pRotData)sprites;

//animated sprite structure required
typedef struct
{
	u16 x;			//x and y position on screen
	u16 y;
	u16 spriteFrame[3];     //animation frame
	int activeFrame;        //which frame is active
}Sprite;

Sprite pacman;		//create instance

//Copy our sprite array to OAM
void CopyOAM()
{
	u16 loop;
	u16* temp;
	temp = (u16*)sprites;
	for(loop = 0; loop < 128*4; loop++)
	{
		OAM[loop] = temp[loop];
	}
}

//Set sprites to off screen
void InitializeSprites()
{
	u16 loop;
	for(loop = 0; loop < 128; loop++)
	{
		sprites[loop].attribute0 = 160;  //y to > 159
		sprites[loop].attribute1 = 240;  //x to > 239
	}
}

//wait for the screen to stop drawing
void WaitForVsync()
{
	while((volatile u16)REG_VCOUNT != 160){}
}

void GetInput()
{
	if(!(*KEYS & KEY_UP))		//if the up key is pressed
	{
		pacman.activeFrame = 0;
		sprites[0].attribute2 = pacman.spriteFrame[pacman.activeFrame];
	}
	if(!(*KEYS & KEY_DOWN))         //if the up down is pressed
	{
 		pacman.activeFrame = 1;
		sprites[0].attribute2 = pacman.spriteFrame[pacman.activeFrame];
	}
	if(!(*KEYS & KEY_LEFT))         //if the up left is pressed
	{
 		pacman.activeFrame = 2;
		sprites[0].attribute2 = pacman.spriteFrame[pacman.activeFrame];
	}
}


int AgbMain()
{
	u16 loop;       //generic loop variable
        pacman.x = 100;	//variables to hold position of pacman sprite on screen
	pacman.y = 60;

       	SetMode(MODE_1 | OBJ_ENABLE | OBJ_MAP_1D); //set mode 2 and enable sprites and 1d mapping

	for(loop = 0; loop < 256; loop++)          //load the palette into memory
		OBJPaletteMem[loop] = palette[loop];

	InitializeSprites();

	sprites[0].attribute0 = COLOR_256 | SQUARE | pacman.y;	//setup sprite info, 256 colour, shape and y-coord
	sprites[0].attribute1 = SIZE_16 | pacman.x;             //size 16x16 and x-coord
	sprites[0].attribute2 = 0;                              //pointer to tile where sprite starts

       	for(loop = 0; loop < 384; loop++)                      //load 1st sprite image data
	{
		OAMData[loop] = pacanimData[loop];
	}

	pacman.activeFrame = 0;                    //set the initial active frame markers
	pacman.spriteFrame[0] = 0;                 //set the frame markers
	pacman.spriteFrame[1] = 8;
	pacman.spriteFrame[2] = 16;

	while(1)                                //main loop
        {
		GetInput();
		WaitForVsync();			//waits for the screen to stop drawing
		CopyOAM();			//Copies sprite array into OAM.
	}
}

