//////////////////////////////////////////////////////////////////////////
// File: 	ch4d.cpp                                            	//
// Description: A program that displays multiple animated sprite	//
// Author:	gbajunkie 	  					//
// Date: 	9th April 2002                                      	//
//////////////////////////////////////////////////////////////////////////

#include "gba.h"        //GBA register definitions
#include "keypad.h"     //button registers
#include "dispcnt.h"    //REG_DISPCNT register #defines
#include "ch4d.h"       //my generic sprite header file
#include "palette.h"  	//256 colour palette array for all sprites
#include "pacanim.h"    //pacman sprite image data
#include "ghostanim.h"  //ghost sprite image data

//create an OAM variable and make it point to the address of OAM
u16* OAM = (u16*)0x7000000;

//create the array of sprites (128 is the maximum)
OAMEntry sprites[128];

//create the rotation and scaling array (overlaps the OAMEntry array memory)
//(dont worry about this for now)
pRotData rotData = (pRotData)sprites;

//animated sprite structure required
typedef struct
{
	u16 x;			//x and y position on screen
	u16 y;
	u16 spriteFrame[3];     //animation frame storage
	int activeFrame;        //which frame is active
	u16 OAMSpriteNum;       //which sprite referring to
}Sprite;

Sprite pacman;		//create pacman sprite instance
Sprite ghost;           //create ghost sprite instance

//Copy sprite array to OAM
void CopyOAM()
{
	u16 loop;
	u16* temp;
	temp = (u16*)sprites;
	for(loop = 0; loop < 128*4; loop++)
	{
		OAM[loop] = temp[loop];
	}
}

//Set sprites to off screen
void InitializeSprites()
{
	u16 loop;
	for(loop = 0; loop < 128; loop++)
	{
		sprites[loop].attribute0 = 160;  //y to > 159
		sprites[loop].attribute1 = 240;  //x to > 239
	}
}

//wait for the screen to stop drawing
void WaitForVsync()
{
	while((volatile u16)REG_VCOUNT != 160){}
}

void GetInput()
{
	if(!(*KEYS & KEY_UP))                   //if the up key is pressed
	{
		pacman.activeFrame = 0;
		sprites[pacman.OAMSpriteNum].attribute2 = pacman.spriteFrame[pacman.activeFrame];
	}
	if(!(*KEYS & KEY_DOWN))                 //if the up down is pressed
	{
 		pacman.activeFrame = 1;
		sprites[pacman.OAMSpriteNum].attribute2 = pacman.spriteFrame[pacman.activeFrame];
	}
	if(!(*KEYS & KEY_LEFT))                 //if the up left is pressed
	{
 		pacman.activeFrame = 2;
		sprites[pacman.OAMSpriteNum].attribute2 = pacman.spriteFrame[pacman.activeFrame];
	}
	if(!(*KEYS & KEY_A))                    //if the A key is pressed
	{
		ghost.activeFrame = 0;
		sprites[ghost.OAMSpriteNum].attribute2 = ghost.spriteFrame[ghost.activeFrame];
	}
	if(!(*KEYS & KEY_B))                    //if the B key is pressed
	{
 		ghost.activeFrame = 1;
		sprites[ghost.OAMSpriteNum].attribute2 = ghost.spriteFrame[ghost.activeFrame];
	}
	if(!(*KEYS & KEY_R))                    //if the R key is pressed
	{
 		ghost.activeFrame = 2;
		sprites[ghost.OAMSpriteNum].attribute2 = ghost.spriteFrame[ghost.activeFrame];
	}
}


int AgbMain()
{
	u16 loop;               //generic loop variable
        pacman.x = 100;		//set coordinates of pacman sprite on screen
	pacman.y = 60;
	ghost.x = 60;   	//set coordinates of ghost sprite on screen
	ghost.y = 100;

       	SetMode(MODE_1 | OBJ_ENABLE | OBJ_MAP_1D); //set mode 1 and enable sprites and 1d mapping

	for(loop = 0; loop < 256; loop++)          //load the palette into memory
		OBJPaletteMem[loop] = palette[loop];

	InitializeSprites();                       //set all sprites off screen (stops artifact)

	sprites[0].attribute0 = COLOR_256 | SQUARE | pacman.y; //setup sprite info, 256 colour, shape and y-coord
	sprites[0].attribute1 = SIZE_16 | pacman.x;            //size 16x16 and x-coord
	sprites[0].attribute2 = 0;                             //pointer to tile where sprite starts

       	for(loop = 0; loop < 384; loop++)       //load 1st sprite image data
	{
		OAMData[loop] = pacanimData[loop];
	}

	pacman.OAMSpriteNum = 0;                //set to the correct sprite number
	pacman.activeFrame = 0;                 //set the initial active frame markers
	pacman.spriteFrame[0] = 0;              //set the frame markers
	pacman.spriteFrame[1] = 8;
	pacman.spriteFrame[2] = 16;

	sprites[1].attribute0 = COLOR_256 | SQUARE | ghost.y;	//setup sprite info, 256 colour, shape and y-coord
	sprites[1].attribute1 = SIZE_16 | ghost.x;              //size 16x16 and x-coord
	sprites[1].attribute2 = 24;                             //pointer to tile where sprite starts

       	for(loop = 384; loop < 768; loop++)    	//load 2nd sprite image data
	{
		OAMData[loop] = ghostanimData[loop-384];
	}

	ghost.OAMSpriteNum = 1;                 //set to the correct sprite number
	ghost.activeFrame = 0;                 	//set the initial active frame markers
	ghost.spriteFrame[0] = 24;              //set the frame markers
	ghost.spriteFrame[1] = 32;
	ghost.spriteFrame[2] = 40;

        while(1)                                //main loop
        {
		GetInput();                     //get input from the GBA buttons
		WaitForVsync();			//waits for the screen to stop drawing
		CopyOAM();			//Copies sprite array into OAM.
	}
}

